def create_config_view(workspace, config_lcv, top_cell_lcv, library_list, view_list, stop_list):
    """Creates a config view in a given lib-cell-view. Sets the top cell lib-cell-view based on given. Sets the config
       library list, view list, and stop list in the global binding section.

    Args:
        workspace (Workspace): A open Skillbridge Workspace object, connected to a specified project virtuoso session.
        config_lcv (list(str str str)): A list containing the library, cell, and view name of where the config will
            be created.
        top_cell_lcv (list(str str str)): A list containing the library, cell, and view name of the Top Cell in the
            created config.
        library_list (str): String of library names separated by spaces.
        view_list (str):  String of view names separated by spaces.
        stop_list (str): String of stops separated by spaces.

    Returns:
         (bool) Returns true is successful, false otherwise
    """
    return workspace['HdbCreateConfigView'](config_lcv, top_cell_lcv, library_list, view_list, stop_list)



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath( '../..'))
# sys.path.append(os.path.relpath())


# -- Project information -----------------------------------------------------
import virtue

project = 'virtue'
copyright = '2020, Cascode-labs'
author = 'Curtis Mayberry'

# The full version, including alpha/beta/rc tags
release = virtue.__version__
print('Release version={num}'.format(num=release))
version = release

# -- General configuration ---------------------------------------------------

# By default Sphinx expects the master doc to be at contents, but ours is index.
master_doc = 'index'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx_copybutton',
    'sphinx_panels',
    "sphinx_github_changelog",
    'sphinx_sitemap',
    'sphinx.ext.autosectionlabel',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

source_suffix = ['.rst']

language = "en"

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "pydata_sphinx_theme"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

panels_add_bootstrap_css = False

html_theme_options = {
    "icon_links": [
        {
            # Label for this link
            "name": "GitHub",
            # URL where the link will redirect
            "url": "https://github.com/cascode-labs/virtue",  # required
            # Icon class (if "type": "fontawesome"), or path to local image (if "type": "local")
            "icon": "fab fa-github-square",
            # The type of image to be used (see below for details)
            "type": "fontawesome",
        },
        {
            "name": "Latest-release",
            "url": "https://github.com/cascode-labs/virtue/releases/latest",  # required
            "icon": "fas fa-tag",
            "type": "fontawesome",
        }
    ],
    "external_links": [
      {"name": "cascode-labs", "url": "http://www.cascode-labs.org/"},
    ],
    "logo": {
        "text": "Virtue",
    },
    "footer_items": ["version", "copyright", "sphinx-version"],
    "show_nav_level": 2,
    "use_edit_page_button": True,
    "google_analytics_id": "G-80SSZFFV2B",
}

html_context = {
    # "github_url": "https://github.com", # or your GitHub Enterprise interprise
    "github_user": "cascode-labs",
    "github_repo": "virtue",
    "github_version": "main",
    "doc_path": "docs",
}

html_baseurl = 'http://www.cascode-labs.org/virtue/'

html_css_files = [
    'css/custom.css',
]



import os
import warnings


def get_project_name_list():
    """ Gets a list of project names sorted in alphabetical order .

    Order example: ['Cat', 'cobra', 'whale', 'Whale', 'zebra']

    Returns:
        List[str]: Directory names at '/prj'.
    """
    return sorted(os.listdir('/prj'), key=str.lower)


def cell_view_exists(workspace, lib_name, cell_name, view_name):
    """Checks if lib-cell-view exists.

    Args:
        workspace (Workspace): A open Skillbridge Workspace object, connected to a specified project virtuoso session.
        lib_name (str): The library name of the view being checked.
        cell_name (str): The cell name of the view being checked.
        view_name (str): The view name of the view being checked.

    Returns:
        bool: True if lib-cell-view exists, False otherwise.
    """
    warnings.simplefilter("ignore")
    result = workspace.db.open_cell_view_by_type(lib_name, cell_name, view_name, "", "r")
    return True if result else False


def get_view_write_path(workspace, lib_name, cell_name, view_name, file_name='text.txt'):
    """Get write path to a specified view.

    Args:
        workspace (Workspace): A open Skillbridge Workspace object, connected to a specified project virtuoso session.
        lib_name (str): The library name of the view being checked.
        cell_name (str): The cell name of the view being checked.
        view_name (str): The view name of the view being checked.
        file_name (:obj:`str`, optional): The file name to be looked for in the specified lib, cell, and view names. Defaults to 'text.txt'.

    Returns:
        str: Write path to the given lib-cell-view, None if file cannot be found.
    """
    return workspace['getWritePathFromLCV'](lib_name, cell_name, view_name, file_name)


def get_library_dict(workspace):
    """Get library names and their related cell and view names.

        Example: {'library_0': {'cell_A': ['view_0', 'view_1'], 'cell_B': []}, 'library_1': {}}

    Args:
        workspace (Workspace): A open Skillbridge Workspace object, connected to a specified project virtuoso session.

    Returns:
        dict(library_name: dict(cell-views))
    """
    result = workspace.dd.get_lib_list()
    lib_cell_view_dict = {library_object.name: _get_cells_in_lib(library_object.cells) for library_object in result}
    return lib_cell_view_dict


def _get_cells_in_lib(cell_objects):
    """ Get cell names and their related view names.

        Example: {'cell_A': ['view_0', 'view_1'], 'cell_B': [], 'cell_C': ['view_0']}
        Example: dict()

    Args:
        cell_objects(List[obj]): List of cell_objects with name and views properties.

    Returns:
        dict(cell_name: List[view_name]) if cell_objects is not None else empty dictionary, dict()
    """
    return {cell_object.name: _get_views_in_cell(cell_object.views) for cell_object in cell_objects} if cell_objects else {}


def _get_views_in_cell(view_objects):
    """Get list of view names.

       Example: ['view_0', 'view_1', 'view_2']
       Example: list()

    Args:
        view_objects(list(object)): List of view objects found in a cell.

    Returns:
        List[view_names] if view_objects is not None else list()
    """
    return [view_object.name for view_object in view_objects] if view_objects else []



def find_all_nets_at_heir_level(workspace, hier_path, top_lib, top_cell, top_view="schematic"):
    """Get list of nets at a certrain hierarchical level in the given lib-cell-veiw.

    Args:
        workspace (Workspace): A open Skillbridge Workspace object, connected to a specified project virtuoso session.
        hier_path (str): The hierarchical level path to search for nets. (example: '/XDUT/' or '/' or '/XDUT/XDUT/')
        top_lib (str): The top library name of the view being parsed.
        top_cell (str): The top cell name of the view being parsed.
        top_view (:obj:`str`, optional): The top view name of the view being parsed. Defaults to 'schematic'.

    Returns:
        list: List of nets found in the given top lib-cell-view at the hierarchical level.
    """
    return workspace['SchFindAllNetsAtHierLevel'](hier_path, topLib=top_lib, topCell=top_cell, topView=top_view)



import os


def get_workspace_id(project_name=None, user_name=None):
    """ Gets the workspace id in the format expected by the Skill side of Skillbridge connection.

    Args:
        project_name(:obj:`str`, optional): Name of project that will a skill bridge workspace will connect to.
          Defaults to the value of 'PROJ_ID' environment variable.
        user_name(:obj:`str`, optional): Name of user the skill bridge connection workspace will be connected to.
          Defaults to the value of 'USER' environment variable.

    Returns:
        str: A workspace id in the format '{project_name}-{user_name}.

    Raises:
        ValueError: An error occurred if the project name or username is not provided and the associated environement
          variable does not exits.
    """
    project_name = get_current_project_name() if not project_name else project_name
    user_name = get_current_user_name() if not user_name else user_name
    if not project_name or not user_name:
        raise ValueError("Need to provide project name and username or be in a project workarea with 'PROJ_ID' and "
                         "'USER' environemnt variables set.")
    return "{prj}-{user}".format(prj=project_name, user=user_name)


def get_current_user_name():
    """Gets current user's username from 'USER' environment variable."""
    return os.getenv('USER')


def get_current_project_name():
    """Gets current user's username from 'PROJ_ID' environment variable."""
    return os.getenv('PROJ_ID')


"""
Run the SKILL tests from Python.  Makes it easy to run them from a 
python IDE.  Not working yet though as skillbridge doesn't currently support 
tables.
"""

from pathlib import Path
from skillbridge import Workspace, Symbol

if __name__ == "__main__":
    tests_path = Path(__file__).parent.resolve()
    run_tests_path = tests_path / "run_tests.ils"
    src_init_path = tests_path /".." / "virtue" / "virtue.init.ils"
    ws = Workspace.open('ids_dev-mayberc')
    print("setup")
    env = env = ws["theEnvironment"]()
    print(env)
    skill_import = ws.__.x
    #oad_result = ws["load"](str(src_init_path))
    #ws["toplevel"](Symbol("ils"))
    print(skill_import)
    test_pkg = skill_import[Symbol("Test")]
    print("run_tests")
    result = test_pkg.run_directory_suite(str(tests_path))
    print(result)